@@ -39,7 +39,7 @@ from nbxmpp.addresses import NoMoreAddresses
 from nbxmpp.tcp import TCPConnection
 from nbxmpp.websocket import WebsocketConnection
 from nbxmpp.smacks import Smacks
-from nbxmpp.auth import SASL
+from nbxmpp.sasl import SASL
 from nbxmpp.const import StreamState
 from nbxmpp.const import StreamError
 from nbxmpp.const import ConnectionType
@@ -212,6 +212,14 @@ class Client(Observable):
     def ignore_tls_errors(self):
         return self._ignore_tls_errors
 
+    @property
+    def tls_version(self):
+        return self._con.tls_version
+
+    @property
+    def ciphersuite(self):
+        return self._con.ciphersuite
+
     def set_ignore_tls_errors(self, ignore):
         self._ignore_tls_errors = ignore
 
@@ -692,7 +700,10 @@ class Client(Observable):
 
         elif self.state == StreamState.AUTH_SUCCESSFUL:
             self._stream_authenticated = True
-            self._start_stream()
+            if self._sasl.is_sasl2():
+                self.state = StreamState.WAIT_FOR_FEATURES
+            else:
+                self._start_stream()
 
         elif self.state == StreamState.AUTH_FAILED:
             self._disconnect_with_error(StreamError.SASL,
@@ -786,7 +797,7 @@ class Client(Observable):
         self.state = StreamState.WAIT_FOR_TLS_PROCEED
 
     def _start_auth(self, features):
-        if not features.has_sasl():
+        if not features.has_sasl() and not features.has_sasl_2():
             self._log.error('Server does not support SASL')
             self._disconnect_with_error(StreamError.SASL,
                                         'sasl-not-supported')
