@@ -25,6 +25,7 @@ from typing import Union
 from typing import Optional
 from typing import cast
 
+import os
 import time
 import hashlib
 import functools
@@ -36,14 +37,20 @@ from dataclasses import asdict
 from gi.repository import GLib
 
 import idna
-from precis_i18n import get_profile
 from nbxmpp.simplexml import Node
 from nbxmpp.namespaces import Namespace
+from nbxmpp.stringprep import nodeprep
+from nbxmpp.stringprep import resourceprep
+from nbxmpp.precis import enforce_precis_username
+from nbxmpp.precis import enforce_precis_opaque
+
 
 def ascii_upper(s):
     return s.upper()
 
+
 SASL_AUTH_MECHS = [
+    'SCRAM-SHA-512-PLUS',
     'SCRAM-SHA-512',
     'SCRAM-SHA-256-PLUS',
     'SCRAM-SHA-256',
@@ -250,7 +257,6 @@ _errorcodes = {
 }
 
 
-_localpart_disallowed_chars = set('"&\'/:<>@')
 _localpart_escape_chars = ' "&\'/:<>@'
 
 
@@ -481,12 +487,17 @@ def validate_localpart(localpart: str) -> str:
     if not localpart or len(localpart.encode()) > 1023:
         raise LocalpartByteLimit
 
-    if _localpart_disallowed_chars & set(localpart):
-        raise LocalpartNotAllowedChar
+    if os.environ.get('NBXMPP_ENFORCE_PRECIS') is None:
+        try:
+            return nodeprep(localpart)
+        except Exception:
+            try:
+                return enforce_precis_username(localpart)
+            except Exception:
+                raise LocalpartNotAllowedChar
 
     try:
-        username = get_profile('UsernameCaseMapped')
-        return username.enforce(localpart)
+        return enforce_precis_username(localpart)
     except Exception:
         raise LocalpartNotAllowedChar
 
@@ -496,9 +507,17 @@ def validate_resourcepart(resourcepart: str) -> str:
     if not resourcepart or len(resourcepart.encode()) > 1023:
         raise ResourcepartByteLimit
 
+    if os.environ.get('NBXMPP_ENFORCE_PRECIS') is None:
+        try:
+            return resourceprep(resourcepart)
+        except Exception:
+            try:
+                return enforce_precis_opaque(resourcepart)
+            except Exception:
+                raise ResourcepartNotAllowedChar
+
     try:
-        opaque = get_profile('OpaqueString')
-        return opaque.enforce(resourcepart)
+        return enforce_precis_opaque(resourcepart)
     except Exception:
         raise ResourcepartNotAllowedChar
 
@@ -520,7 +539,7 @@ def validate_domainpart(domainpart: str) -> str:
         domainpart = domainpart[:-1]
 
     try:
-        idna_encode(domainpart)
+        domainpart = idna2008_prep(domainpart)
     except Exception:
         raise DomainpartNotAllowedChar
 
@@ -528,8 +547,18 @@ def validate_domainpart(domainpart: str) -> str:
 
 
 @functools.lru_cache(maxsize=None)
-def idna_encode(domain: str) -> str:
-    return idna.encode(domain, uts46=True).decode()
+def idna2008_prep(domain: str, to_ascii: bool = False) -> str:
+    '''
+    Prepare with UTS46 case mapping to stay compatibel with the IDNA2003
+    mapping. Further try to encode the domain to catch illegal domains.
+    Only return the case mapped domain because on the XMPP wire,UTF8 domains
+    are fine.
+    '''
+    domain = idna.uts46_remap(domain)
+    encoded_domain = idna.encode(domain)
+    if to_ascii:
+        return encoded_domain.decode()
+    return domain
 
 
 @functools.lru_cache(maxsize=None)
@@ -685,7 +714,7 @@ class JID:
         return not self.__eq__(other)
 
     def domain_to_ascii(self) -> str:
-        return idna_encode(self.domain)
+        return idna2008_prep(self.domain, to_ascii=True)
 
     @property
     def bare(self) -> str:
@@ -1708,10 +1737,15 @@ class Features(Node):
     def has_sasl(self):
         return self.getTag('mechanisms', namespace=Namespace.SASL) is not None
 
+    def has_sasl_2(self):
+        return self.getTag('mechanisms', namespace=Namespace.SASL2) is not None
+
     def get_mechs(self) -> set[str]:
-        mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL)
+        mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL2)
         if mechanisms is None:
-            return set()
+            mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL)
+            if mechanisms is None:
+                return set()
         mechanisms = mechanisms.getTags('mechanism')
         return set(mech.getData() for mech in mechanisms)
 
