@@ -17,6 +17,7 @@
 
 import logging
 from collections import deque
+from typing import Optional
 
 from gi.repository import GLib
 from gi.repository import Gio
@@ -26,6 +27,7 @@ from nbxmpp.const import TCPState
 from nbxmpp.const import ConnectionType
 from nbxmpp.util import utf8_decode
 from nbxmpp.util import convert_tls_error_flags
+from nbxmpp.util import min_version
 from nbxmpp.connection import Connection
 
 log = logging.getLogger('nbxmpp.tcp')
@@ -62,6 +64,28 @@ class TCPConnection(Connection):
 
         self._keepalive_id = None
 
+    @property
+    def tls_version(self) -> Optional[int]:
+        if self._con is None:
+            return None
+
+        if not min_version('GLib', '2.69.0'):
+            return None
+
+        tls_con = self._con.get_base_io_stream()
+        return tls_con.get_protocol_version()
+
+    @property
+    def ciphersuite(self) -> Optional[str]:
+        if self._con is None:
+            return None
+
+        if not min_version('GLib', '2.69.0'):
+            return None
+
+        tls_con = self._con.get_base_io_stream()
+        return tls_con.get_ciphersuite_name()
+
     def connect(self):
         self.state = TCPState.CONNECTING
 
@@ -233,11 +257,8 @@ class TCPConnection(Connection):
 
         data = data.get_data()
         if not data:
-            if self._state == TCPState.DISCONNECTING:
-                self._log.info('Reveived zero data on _read_async()')
-                self._finalize('disconnected')
-            else:
-                self._log.warning('Reveived zero data on _read_async()')
+            self._log.info('Reveived zero data on _read_async()')
+            self._finalize('disconnected')
             return
 
         self._renew_keepalive_timer()
